from django import forms

# import models
from userextensions.models import (UserPreference, ServiceAccount)


class UserPreferenceForm(forms.ModelForm):
    """ Form class used to add/edit UserPreference objects """
    class Meta:
        model = UserPreference
        exclude = ['created_at', 'updated_at', 'user']
        widgets = {
            'recents_count': forms.NumberInput(attrs={'class': 'form-control'}),
            'page_refresh_time': forms.NumberInput(attrs={'class': 'form-control'}),
            'theme': forms.Select(attrs={'class': 'form-control'}),
            'start_page': forms.TextInput(attrs={'class': 'form-control'}),
        }


class ServiceAccountForm(forms.ModelForm):
    """ Form class used to add/edit ServiceAccount objects """
    class Meta:
        model = ServiceAccount
        exclude = ['created_at', 'updated_at', 'user']
        widgets = {
            'group': forms.Select(attrs={'class': 'form-control'}),
            'description': forms.TextInput(attrs={'class': 'form-control'}),
            'enabled': forms.CheckboxInput(attrs={'class': 'form-control'}),
        }



import re
from django.http import JsonResponse
from rest_framework import status
from .models import ServiceAccount


class ServiceAccountControlMixin:
    """ A mixin for Django Rest Framework viewsets that checks if the request is made from a ServiceAccount and only
    allows access to the endpoint if the ServiceAccount is enabled and admin_enabled. If the ServiceAccount is disabled
    by the owner or an admin, a 403 error will be received instead of the API response. This currently works for APIs
    using TokenAuthentication (rest_framework.authentication.TokenAuthentication). """
    def dispatch(self, request, *args, **kwargs):
        mo = re.search('Token (\S+)', request.META.get('HTTP_AUTHORIZATION', ''))
        if mo:
            srv_acct = ServiceAccount.objects.get_object_or_none(user__auth_token__key=mo.group(1))
            if srv_acct:
                if srv_acct.admin_enabled is False:
                    return JsonResponse(data={'detail': 'this service account has been administratively disabled'},
                                        status=status.HTTP_403_FORBIDDEN)
                elif srv_acct.enabled is False:
                    return JsonResponse(data={'detail': 'this service account has been disabled'},
                                        status=status.HTTP_403_FORBIDDEN)
        return super().dispatch(request, *args, **kwargs)


class AllowOnlyServiceAccountMixin:
    """ A mixin for Django Rest Framework viewsets that only allows responses to requests made from ServiceAccounts.
    This currently works for APIs using TokenAuthentication (rest_framework.authentication.TokenAuthentication)."""
    def dispatch(self, request, *args, **kwargs):
        mo = re.search('Token (\S+)', request.META.get('HTTP_AUTHORIZATION', ''))
        if mo:
            srv_acct = ServiceAccount.objects.get_object_or_none(user__auth_token__key=mo.group(1))
            if not srv_acct:
                return JsonResponse(data={'detail': 'access to this endpoint is only available to service accounts'},
                                    status=status.HTTP_403_FORBIDDEN)
        return super().dispatch(request, *args, **kwargs)


class AllowOnlyEnabledServiceAccountMixin:
    """ A mixin for Django Rest Framework viewsets that checks if the request is made from a ServiceAccount and only
    allows access to the endpoint if an enabled ServiceAccount made the request. If the ServiceAccount is disabled
    by the owner or an admin, a 403 error will be received instead of the API response. This currently works for APIs
    using TokenAuthentication (rest_framework.authentication.TokenAuthentication). """
    def dispatch(self, request, *args, **kwargs):
        mo = re.search('Token (\S+)', request.META.get('HTTP_AUTHORIZATION', ''))
        if mo:
            srv_acct = ServiceAccount.objects.get_object_or_none(user__auth_token__key=mo.group(1))
            if srv_acct:
                if srv_acct.admin_enabled is False:
                    return JsonResponse(data={'detail': 'this service account has been administratively disabled'},
                                        status=status.HTTP_403_FORBIDDEN)
                elif srv_acct.enabled is False:
                    return JsonResponse(data={'detail': 'this service account has been disabled'},
                                        status=status.HTTP_403_FORBIDDEN)
            else:
                return JsonResponse(data={'detail': 'access to this endpoint is only available to enabled '
                                                    'service accounts'}, status=status.HTTP_403_FORBIDDEN)
        return super().dispatch(request, *args, **kwargs)



from django.urls import path
from userextensions import views
from userextensions.views import ajax
from userextensions.views import action

app_name = 'userextensions'

urlpatterns = [

    # list views
    path('list_recents/', views.ListRecents.as_view(), name='list_recents'),
    path('list_favorites/', views.ListFavorites.as_view(), name='list_favorites'),

    # detail views
    path('detail_user/', views.DetailUser.as_view(), name='detail_user'),

    # create views
    path('add_favorite/', views.AddFavorite.as_view(), name='add_favorite'),

    # update views
    path('set_start_page/', views.SetStartPage.as_view(), name='set_start_page'),

    # delete views
    path('delete_favorite/<int:pk>', views.DeleteFavorite.as_view(), name='delete_favorite'),
    path('delete_recent/<int:pk>', views.DeleteRecent.as_view(), name='delete_recent'),

    # custom views
    path('user_login_redirect/', views.UserLoginRedirect.as_view(), name='user_login_redirect'),
    path('manage_service_accounts/', views.ManageServiceAccounts.as_view(), name='manage_service_accounts'),

    # action views
    path('refresh_api_token', views.RefreshApiToken.as_view(), name='refresh_api_token'),
    path('refresh_srv_acct_token', views.RefreshSrvAcctApiToken.as_view(), name='refresh_srv_acct_token'),
    path('create_srv_account', action.CreateServiceAccount.as_view(), name='create_srv_account'),
    path('delete_srv_account', action.DeleteServiceAccount.as_view(), name='delete_srv_account'),
    path('enable_srv_account', action.EnableServiceAccount.as_view(), name='enable_srv_account'),
    path('disable_srv_account', action.DisableServiceAccount.as_view(), name='disable_srv_account'),
    path('edit_favorite', action.EditFavorite.as_view(), name='edit_favorite'),

    # ajax views
    path('get_users_per_group', ajax.get_users_per_group, name='get_users_per_group'),
    path('get_srv_acct_token_history', ajax.get_srv_acct_token_history, name='get_srv_acct_token_history'),
    path('show_srv_acct_token', ajax.show_srv_acct_token, name='show_srv_acct_token'),

]



"""

"""
import re
from django.conf import settings
from django.contrib.auth.models import User, Group
from django.db import models
from django.core.exceptions import ValidationError

from handyhelpers.managers import HandyHelperModelManager
from handyhelpers.models import HandyHelperBaseModel
from rest_framework.authtoken.models import Token


class Theme(HandyHelperBaseModel):
    """ This model tracks themes. It can be used to provide user preferred frontend styling options based on
    defined css files. """
    name = models.CharField(max_length=32, unique=True, help_text='name of theme')
    css_file = models.CharField(max_length=255, unique=True, blank=True, null=True,
                                help_text='path to css file for theme')

    def __str__(self):
        return self.name


class UserPreference(HandyHelperBaseModel):
    """ This table tracks user preferences. Fields include theme, recents_count, page_refresh_time, and start_page. """
    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name='preference')
    theme = models.ForeignKey(Theme, blank=True, null=True, help_text='theme to use for web pages',
                              on_delete=models.CASCADE)
    recents_count = models.IntegerField(default=25, blank=True, null=True,
                                        help_text='number of recents to keep a record of')
    page_refresh_time = models.IntegerField(default=5, blank=True, null=True,
                                            help_text='time, in minutes, to auto-refresh a page (where applicable')
    start_page = models.CharField(max_length=255, blank=True, null=True, help_text='url to redirect to after login')

    def __str__(self):
        return self.user.username


class UserRecent(HandyHelperBaseModel):
    """ This table stored recently visited urls. """
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='recent')
    url = models.URLField(help_text='url endpoint')

    class Meta:
        unique_together = (('url', 'user'), )

    def __str__(self):
        return self.url


class UserFavorite(HandyHelperBaseModel):
    """ This table stores user-defined favorites. """
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='favorite')
    name = models.CharField(max_length=32, blank=True, null=True, help_text='name/label/reference for this favorite')
    url = models.URLField(help_text='url endpoint')

    class Meta:
        unique_together = (('url', 'user'), )

    def __str__(self):
        return self.url


class ServiceAccount(HandyHelperBaseModel):
    """ This table stores service accounts and maps to a (service account) user and group """
    user = models.OneToOneField(User, blank=True, null=True, on_delete=models.CASCADE)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    enabled = models.BooleanField(default=True, help_text='owner enable/disable state of service account')
    admin_enabled = models.BooleanField(default=True, help_text='admin enable/disable state of service account')
    description = models.CharField(max_length=254, blank=True, null=True, help_text='optional description')

    class Meta:
        unique_together = (('user', 'group'), )

    def __str__(self):
        return self.user.username

    def clean(self):
        """ clean/update/validate data before saving """
        if not self.user:
            # If the SRV_ACCOUNT_GROUP_FILTER_LIST (list) settings variable is set, search for a matching regex in the
            # group name. Use the first match found. If no match is found or an exception is encountered, use the
            # group name as is.
            try:
                group_name = None
                for regex in getattr(settings, 'SRV_ACCOUNT_GROUP_FILTER_LIST', list()):
                    match = re.match(regex, self.group.name)
                    if match:
                        group_name = match.group(1)
                        break
                if not group_name:
                    group_name = self.group.name
            except Exception as err:
                group_name = self.group.name

            # Create name based on prefix + group + suffix; prefix and/or suffix is required. The prefix & suffix values
            # are provided by django settings variables. If neither is set, a default suffix '_srv' will be used.
            prefix = getattr(settings, 'SRV_ACCOUNT_PREFIX', '')
            suffix = getattr(settings, 'SRV_ACCOUNT_SUFFIX', '')
            if not prefix and not suffix:
                suffix = '_srv'
            username = prefix + group_name + suffix
        else:
            username = self.user.username

        # before creating the service account, we need to get_or_create the user
        self.user = User.objects.get_or_create(username=username)[0]
        # add now add the new user to the matching group
        self.group.user_set.add(self.user)

        # check if multiple service accounts per group are allowed. This is set in the django settings file with
        # the ALLOW_MULTIPLE_SRV_ACCOUNTS variable. By default, only one service account per group is allowed. To
        # enabled multiple service accounts per group, set SRV_ALLOW_MULTIPLE_ACCOUNTS=True
        if not self.pk:
            allow_multiple = getattr(settings, 'ALLOW_MULTIPLE_SRV_ACCOUNTS', False)
            existing = ServiceAccount.objects.filter(group=self.group)
            if existing and allow_multiple is False:
                raise ValidationError({'group': 'Multiple service accounts per group is currently not allowed. To '
                                                'enable multiple accounts per group, set '
                                                'ALLOW_MULTIPLE_SRV_ACCOUNTS=True in the django settings. '})

    def save(self, *args, **kwargs):
        self.full_clean()
        super().save(*args, **kwargs)

    def create_drf_token(self):
        """ create a drf token for this service account """
        Token.objects.get_or_create(user=self.user)


class ServiceAccountTokenHistory(models.Model):
    """ This table track changes to a ServiceAccount API token refresh history """
    objects = HandyHelperModelManager()
    actor = models.ForeignKey(User, null=True, on_delete=models.SET_NULL, help_text='user who refreshed token')
    srv_acct = models.ForeignKey(ServiceAccount, on_delete=models.CASCADE, help_text='')
    timestamp = models.DateTimeField(auto_now_add=True, help_text='date/time when this token refresh occurred')

    def __str__(self):
        return self.srv_acct.user.username



# Generated by Django 2.2.6 on 2019-12-12 05:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('userextensions', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='userpreference',
            name='start_page',
            field=models.CharField(blank=True, help_text='url to redirect to after user login', max_length=255, null=True),
        ),
    ]



# Generated by Django 2.2.15 on 2020-11-14 05:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('userextensions', '0004_serviceaccount_serviceaccounttokenhistory'),
    ]

    operations = [
        migrations.AddField(
            model_name='serviceaccount',
            name='admin_enabled',
            field=models.BooleanField(default=True, help_text='admin enable/disable state of service account'),
        ),
        migrations.AlterField(
            model_name='serviceaccount',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='serviceaccount',
            name='enabled',
            field=models.BooleanField(default=True, help_text='owner enable/disable state of service account'),
        ),
        migrations.AlterField(
            model_name='serviceaccount',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='theme',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='theme',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='userfavorite',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='userfavorite',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='userpreference',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='userpreference',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='userrecent',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='userrecent',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



"""

"""
from django.utils import timezone
from django.conf import settings
from django.urls import resolve
from urllib.parse import urlparse

# Use MiddlewareMixin when present (Django >= 1.10)
try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object

# import models
from userextensions.models import UserRecent


class UserRecentsMiddleware(MiddlewareMixin):
    """ This middleware parses data from requests and, where applicable, stores the full url path in the request to the
    users list of recently viewed pages (recents). No recents will be stored if the user is not authenticated, user can
    not be determined, or if the URL is invalid.

    Tracking can be filtered byt methods, URL prefixes, and static URLs
    via parameters in the settings.py file. The following are configurable:

        method: tracks only specified methods; defaults to GET
                TRACK_METHOD_LIST = ['GET', ]

        URL prefixes: will not track URLs that start with the specified prefixes
                SKIP_URL_PREFIX_LIST = ['/admin/', '/__debug__/']

        static URLs: will not track the specified URLs
                SKIP_FIXED_URL_LIST = ['/', '/login/', '/logout/', ]
    """
    def process_request(self, request):
        """ read user and url (path) from request, if valid and not in a skip list, add to recents """
        # only track specified methods
        track_method_list = getattr(settings, 'TRACK_METHOD_LIST', ['GET'])
        if request.method not in track_method_list:
            return

        # do not track url if user is not authenticated
        if not request.user.is_authenticated:
            return
        if not getattr(request, 'user'):
            return

        # do not track if url (full path) can not be determined
        path = getattr(request, 'get_full_path')()
        if not path:
            return

        # do not track admin or debug urls
        skip_url_prefix_list = getattr(settings, 'SKIP_URL_PREFIX_LIST', ['/admin/', '/__debug__/'])
        for prefix in skip_url_prefix_list:
            if path.startswith(prefix):
                return

        # do not track urls in the skip_fixed_url_list
        skip_fixed_url_list = getattr(settings, 'SKIP_FIXED_URL_LIST', ['/', '/login/', '/logout/',
                                                                        '/userextensions/list_recents/',
                                                                        '/userextensions/user_login_redirect/'])
        for url in skip_fixed_url_list:
            if path == url:
                return

        # do not track invalid urls
        try:
            resolve(urlparse(path).path)
        except:
            return

        # add recent
        UserRecent.objects.update_or_create(url=request.get_full_path(),
                                            user=request.user,
                                            defaults=dict(url=request.get_full_path(),
                                                          user=request.user,
                                                          updated_at=timezone.now())
                                            )



"""
Views used specifically for handling AJAX Requests on Amazon objects
"""
# import system modules
import json

# import django modules
from django.http import HttpResponse
from django.template import loader
from django.views.decorators.http import require_GET

# import models
from django.contrib.auth.models import Group
from userextensions.models import ServiceAccountTokenHistory


@require_GET
def get_users_per_group(request):
    """
    Description:
        Get all users in a group.
    Args:
        request: AJAX request object.
    Returns:
        HttpResponse: JSON formatted response.
    """
    if (request.is_ajax()) and (request.method == 'GET'):
        if 'client_response' in request.GET:
            obj_id = request.GET['client_response']
            obj = Group.objects.get(id=obj_id)
            template = loader.get_template('userextensions/ajax/get_users_per_group.htm')
            return HttpResponse(json.dumps({'server_response': template.render(
                {'queryset': obj.user_set.all().order_by('username')})}),
                                content_type='application/javascript')
        else:
            return HttpResponse('Invalid request inputs', status=400)
    else:
        return HttpResponse('Invalid request', status=400)


@require_GET
def get_srv_acct_token_history(request):
    """
    Description:
        Get the API token refresh history for a give ServiceAccount.
    Args:
        request: AJAX request object.
    Returns:
        HttpResponse: JSON formatted response.
    """
    if (request.is_ajax()) and (request.method == 'GET'):
        if 'client_response' in request.GET:
            obj_id = request.GET['client_response']
            queryset = ServiceAccountTokenHistory.objects.filter(srv_acct__id=obj_id).order_by('-timestamp')
            template = loader.get_template('userextensions/ajax/get_token_history_for_srv_acct.htm')
            return HttpResponse(json.dumps({'server_response': template.render({'queryset': queryset})}),
                                content_type='application/javascript')
        else:
            return HttpResponse('Invalid request inputs', status=400)
    else:
        return HttpResponse('Invalid request', status=400)


@require_GET
def show_srv_acct_token(request):
    """
    Description:
        show the API token for a service account
    Args:
        request: AJAX request object.
    Returns:
        HttpResponse: JSON formatted response.
    """
    if (request.is_ajax()) and (request.method == 'GET'):
        if 'client_response' in request.GET:
            token = request.GET['client_response']
            template = loader.get_template('userextensions/ajax/show_srv_acct_token.htm')
            return HttpResponse(json.dumps({'server_response': template.render({'token': token})}),
                                content_type='application/javascript')
        else:
            return HttpResponse('Invalid request inputs', status=400)
    else:
        return HttpResponse('Invalid request', status=400)




from .models import ServiceAccount


def enforce_srv_account_enabled(func):
    """
    Decorator used on an API intended to enforce service account enabled status. If use attempting action is NOT a
    service account, proceed normally.
    If the service account has enabled = False OR admin_enabled = False, attempted action will blocked. Otherwise,
    proceed normally.
    """
    pass



"""
django-userextensions library
----------------------------
A user extension module for django.
:copyright: (c) 2019 by David Slusser
:license: GPL-3.0. See LICENSE for more details
"""

__title__ = 'django-userextensions'
__version__ = '0.1.1'
__author__ = 'David Slusser'
__email__ = 'dbslusser@gmail.com'
__license__ = 'GPL-3.0'
__copyright__ = 'Copyright 2019 David Slusser'


default_app_config = 'userextensions.apps.UserExtensionsConfig'



# Generated by Django 2.2.15 on 2020-10-16 04:03

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0011_update_proxy_permissions'),
        ('userextensions', '0003_auto_20200117_2153'),
    ]

    operations = [
        migrations.CreateModel(
            name='ServiceAccount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='date/time when this row was first created')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='date/time this row was last updated')),
                ('enabled', models.BooleanField(default=True, help_text='enable/disable state of user')),
                ('description', models.CharField(blank=True, help_text='optional description', max_length=254, null=True)),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
                ('user', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('user', 'group')},
            },
        ),
        migrations.CreateModel(
            name='ServiceAccountTokenHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(auto_now_add=True, help_text='date/time when this token refresh occurred')),
                ('actor', models.ForeignKey(help_text='user who refreshed token', null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
                ('srv_acct', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='userextensions.ServiceAccount')),
            ],
        ),
    ]



# Generated by Django 3.2 on 2021-05-07 18:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('userextensions', '0005_auto_20201114_0548'),
    ]

    operations = [
        migrations.AlterField(
            model_name='serviceaccount',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='serviceaccounttokenhistory',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='theme',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='userfavorite',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='userpreference',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='userrecent',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



import os
from setuptools import setup, find_packages
import userextensions

with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as readme:
    README = readme.read()


with open('requirements.txt') as f:
    required = f.read().splitlines()

version = userextensions.__version__

setup(
    name='django-userextensions',
    description='A user extension module for django',
    long_description=README,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    include_package_data=True,
    version=version,
    license=userextensions.__license__,
    author=userextensions.__author__,
    author_email=userextensions.__email__,
    url='https://github.com/davidslusser/django-userextensions',
    download_url='https://github.com/davidslusser/django-userextensions/archive/{}.tar.gz'.format(version),
    keywords=['django', 'helpers', 'extension', 'user', 'profile'],
    install_requires=required,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Framework :: Django :: 2.2',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



# Generated by Django 2.2.9 on 2020-01-17 21:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('userextensions', '0002_userpreference_start_page'),
    ]

    operations = [
        migrations.AlterField(
            model_name='theme',
            name='css_file',
            field=models.CharField(blank=True, help_text='path to css file for theme', max_length=255, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='userpreference',
            name='start_page',
            field=models.CharField(blank=True, help_text='url to redirect to after login', max_length=255, null=True),
        ),
    ]
